/** MID811 API. API functions for the 811 board.
 * Copyright 2009-2021 Sensoray Company Inc.
 * This functions encapsulate the DirectShow functionality of the 811 AVStream driver
 * These functions are not required for developing an application. If the user
 * wants to use a different DirectShow program(AMCap) or write their own DirectShow program, they may do so.
 * Source code for this API is available for Sensoray customers only.
 *  contact support@sensoray.com for password
 *  Note: No support will be given for compiling or modifying the source code of this DLL API.
 */

#ifndef _MID811FUNC_H
#define _MID811FUNC_H

#define MAX_DEVICES 32  // maximum number of 811s in system

#include "mid811types.h"

#define VERSION_MID811_TEXT "1.0.7"
/** Opens the SDK/API
  * @return 0 on success
  */
MID811_API int __stdcall S811_Open(void);

/** Cleans up and shuts down the API gracefully
 *  @return 0 on success
 */
MID811_API int __stdcall S811_Close(void); 


// query number of devices in system
MID811_API int __stdcall S811_GetNumDevices(int *pNumDevices);


/** Sets the video input source. 
 *  @param input is the enumerated input MID811_SOURCE enum(composite, svideo)
 *  @return 0 on success
 */
MID811_API int __stdcall S811_SetSource(MID811_SOURCE input, int devid);

/** Gets the current video source
 *  @param pSource pointer to returned value of current input source
 *  @return 0 on success 
*/
MID811_API int __stdcall S811_GetSource(MID811_SOURCE *pSource, int devid );

/** set the current video system.  If board is currently streaming 
 *  board will be stopped and restarted.  If board recording, recording will
 *  be stopped.
 *  @param vidsys is the video system type
 *  @return 0 on success
 */
MID811_API int __stdcall S811_SetVidSys(MID811_VIDSYS vidsys, int devid );

MID811_API int __stdcall S811_GetVidSys(MID811_VIDSYS *vidsys, int devid);


/** retrieves current SDK and file information
 *  @return 0 on success, negative otherwise 
 */
MID811_API int __stdcall S811_GetStatus(MID811STATUS *pStatus, int devid );
//unicode version
MID811_API int __stdcall S811_GetStatusW(MID811STATUS_W *pStatus, int devid );


/** starts the preview of video and audio(if selected)
  * @param devid is the board/channelid instance in the system
  */
MID811_API int __stdcall S811_StartPreview(int devid);

/** stops video/audio stream and stops recording if recording
 */
MID811_API int __stdcall S811_StopPreview(int devid);

/** set the recording mode
 *  @param recMode MID811_REC setting
 *  @return 0 on success
 */
MID811_API int __stdcall S811_SetRecordMode(MID811_REC recMode, int devid);

/** get the recording mode
 *  @param recMode MID811_REC setting
 *  @return 0 on success
 */
MID811_API int __stdcall  S811_GetRecordMode(MID811_REC *recMode, int devid);

/** 
 * Starts recording to file using filter graph previously set up.
 * @param filename is the video/mux filename to record to.  Use the full path and extension
 * @return 0 on success
 */
MID811_API int __stdcall S811_StartRecord(const char *filename, int devid );
// unicode
MID811_API int __stdcall S811_StartRecordW(const wchar_t *filename, int devid );

/** pause recording
 */
MID811_API int __stdcall S811_PauseRecord	(int devid );

/** resume recording
 */
MID811_API int __stdcall S811_ResumeRecord	(int devid );

/** stops recording to file. video/sound preview window will continue
 */
MID811_API int __stdcall S811_StopRecord(int devid );



/////////////////////////////////////////////////
// General configuration options

/** set the brightness, contrast and saturation
*   @param param is the parameter to change(MID811_LEVEL_CONTRAST, MID811_LEVEL_BRIGHTNESS, MID811_LEVEL_SATURATION, MID811_LEVEL_HUE)
*   @param value is the value of the parameter
*    ( BRIGHTNESS: 0 to 255 default 128)
*    ( SATURATION: -127 to 127 default 64)
*    ( CONTRAST: -127 to 127 default 68 (negative values not recommended))
*    ( HUE: -127 to 127, default 0 (recommended to stay unchanged))
*   @return 0 on success
*/
MID811_API int __stdcall S811_SetLevels(int param, int value, int devid );

/** get the brightness, contrast or saturation
  * @param param is the parameter to get(MID811_LEVEL_CONTRAST, MID811_LEVEL_BRIGHTNESS, MID811_LEVEL_SATURATION, MID811_LEVEL_HUE)
  * @param value is the returned value of the parameter
  * @return 0 on success
  */
MID811_API int __stdcall S811_GetLevel(int param, int *value, int devid );


/** 
 *  Mutes the audio on the host computer.  Audio will still be recorded.  
 *  Note: if currently streaming(StartStream used), this function will stop the stream and restart it.
 *  Default DLL mode is to have audio muted
 *  @param bMute whether to mute audio on PC
 *  @return 0 on success
 */
MID811_API int __stdcall S811_SetMute(BOOL bMute, int devid);

/** 
 * Gets the mute setting
 *
 * @param bMute pointer to retrieved mute setting
 * @return 0 on success 
 */
MID811_API int __stdcall S811_GetMute(BOOL *bMute, int devid);

/** Set the video size.  If stream is running, this call will stop and restart it
 *  @param enum eVidSize see vidsizes( 0-- full size default, 1--half size(352x240 for NTSC))
 *  @return 0 on success
 */
MID811_API int __stdcall S811_SetVidSize(MID811_RES eVidSize, int devid);

MID811_API int __stdcall S811_GetVidSize(int *iVidSize, int devid);

/** SetRenderVideo allows one to turn off the video window while recording
 *  @param bDisplayVideo whether to display video or not
 *  @return 0 on success
 */
MID811_API int __stdcall S811_SetRenderVideo(BOOL bDisplayVideo, int devid);
/** Gets the display video setting
 *  @param bDisplayVideo pointer to display video setting
 *  @return 0 on success
 */
MID811_API int __stdcall S811_GetRenderVideo(BOOL *bDisplayVideo, int devid);


////////////////////////////////////////////////////////////////////////
// Callback functions from application
/** Repaint callback should be called when application receives a WM_PAINT event.  This is necessary to
 *  notify the video renderer of a repaint event. 
 *  @param hdc device context handle or NULL if unknown
 *  @return 0 on success
 */
MID811_API int __stdcall S811_Repaint( HDC hdc);

/** Informs the video renderer that a WM_DISPLAYCHANGE message has been received by the application
 *  This callback should be called when the application has a WM_DISPLAYCHANGE event
 *  @return 0 on success
 */
MID811_API int __stdcall S811_DisplayChange();

// end callback functions
////////////////////////////////////////////////////////////////////////

/** Set the window for the video streaming per device
 *  @param hwnd pointer to video display window
 *  @param devid: device id(board and channel)
 *  Note: this setting will override S811_SetRenderVideo and turn on 
 *  Video rendering(display) as it is assumed setting a window means display is wanted
 */
MID811_API int __stdcall S811_SetPreviewWindow( HWND hwnd, int devid);


/** set the video position in the clipping window 
 *  @param xpos x position of video
 *  @param ypos y position of video
 *  @param xsize size of video display
 *  @param ysize size of video display
 *  @param left_clip number of pixels at left of source video to clip
 *  @param top_clip y number of pixels at top of source video to clip
 *  @param right_clip number of pixels at right of source video to clip
 *  @param bottom_clip number of pixels at bottom of source video to clip
 *  @return 0 on success
 */ 
MID811_API int __stdcall S811_SetVideoPosition(int xpos, int ypos, int xsize, int ysize, int left_clip, int top_clip, int right_clip, int bottom_clip, int devid);


// Enable Snapshots allows the user to turn on snapshot capabilities.
// Snapshots off by default for CPU performance.
MID811_API int __stdcall S811_EnableSnapshots( BOOL bOnOff, int devid);


// set colorspace for preview mode
MID811_API int __stdcall S811_SetColorSpace(MID811_COLORSPACE colorspace, int devid);


// returns frame in pFrame.
MID811_API int __stdcall S811_GetFrame( long bufSize, unsigned char *pFrame, BITMAPINFOHEADER *pBMI, int devid);

/** takes a snapshot and saves to file in filename.  
*  Note : JPEGs are compressed by SW in Windows, not by the hardware.
*         BMPs will be more CPU efficient but take more space
*   @param szFilename is the fully qualified file + path( without extension)
*   @param filetype is the file type to save(MID811_FILE_JPEG and/or MID811_FILE_BMP)
*   @param qual jpeg quality, (use 25-100, default 100).
*   @param devid: device id(board and channel)
*/ 
MID811_API int __stdcall S811_SnapshotToFile( const char *szFilename, int filetype, int qual, int devid);
// unicode version
MID811_API int __stdcall S811_SnapshotToFileW( const wchar_t *szFilename, int filetype, int qual, int devid);

// retrieves native size of captured image
MID811_API int __stdcall S811_GetVideoSize( RECT *rect, int devid);

MID811_API int __stdcall S811_StartCaptureOnly(int mode, int devid);
MID811_API int __stdcall S811_StopCaptureOnly(int devid);

// callback functions for capturing to memory.

MID811_API int __stdcall S811_StartCallback(int colorspace, int devid);
MID811_API int __stdcall S811_StopCallback(int devid);
typedef int (__stdcall *cbfunc_t) ( BYTE *data, long size, int mode, int devid);
MID811_API int __stdcall S811_RegisterVidCB( cbfunc_t callback, int devid);



#endif
